#!/usr/bin/python
# encoding=utf-8

import json
from typing import Any
from urllib.parse import unquote

import httpx
import requests
import urllib3
from loguru import logger
from requests.structures import CaseInsensitiveDict

from tep.libraries.Result import Result
from tep.libraries.TepResponse import TepResponse

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

_template = """\n
        URL: {url}
        Method: {method}
        Headers: {headers}
        Request Body: {request_body}
        Status Code: {status_code}
        Response Body: {response_body}
        Elapsed: {elapsed}s
        """


def HTTPRequestImpl(method, url, **kwargs) -> Result:
    use_http2 = kwargs.pop("http2", False)
    if use_http2:
        return _http2(method, url, **kwargs)
    return _http1(method, url, **kwargs)


def _http1(method, url, **kwargs):
    """
    Requests.request native usage, adding response callbacks
    """
    response = requests.request(
        method, url,
        hooks={'response': _response_callback},
        **kwargs
    )
    result = Result()
    result.response = TepResponse(response)
    return result


def _http2(method, url, **kwargs) -> Result:
    with httpx.Client(event_hooks={'response': [_response2_callback]}) as client:
        r = client.request(method, url, **kwargs)
        result = Result()
        result.response = r
        return result


def _response_callback(response, *args, **kwargs):
    """
    Response callback, recording request meta information
    """
    log = _template.format(
        url=unquote(response.request.url),
        method=response.request.method,
        headers=_json_text(response.request.headers),
        request_body=_json_text(response.request.body),
        status_code=response.status_code,
        response_body=response.text,
        elapsed=response.elapsed.total_seconds()
    )
    logger.info(log)


def _response2_callback(response: httpx.Response):
    """
    Response callback, recording request meta information
    """
    response.read()
    log = _template.format(
        url=response.request.url,
        method=response.request.method,
        headers=_json_text(response.request.headers),
        request_body=_json_text(response.request.content),
        status_code=response.status_code,
        response_body=response.text,
        elapsed=response.elapsed.total_seconds()
    )
    logger.info(log)


def _json_text(o: Any) -> [str, Any]:
    try:
        if not o:
            return o
        elif isinstance(o, bytes):
            return json.dumps(json.loads(o.decode("utf-8")), ensure_ascii=False)
        elif isinstance(o, dict) or isinstance(o, list) or isinstance(o, str):
            return json.dumps(o, ensure_ascii=False)
        elif isinstance(o, CaseInsensitiveDict):
            return json.dumps(dict(o), ensure_ascii=False)
        else:
            logger.warning("JSON parse unknown type: {}", type(o))
            return o
    except Exception as e:
        logger.error("JSON parse exception: {}", e)
        return o



#!/usr/bin/python
# encoding=utf-8

import os

from tep.libraries.Config import Config
from tep.libraries.File import File
from tep.libraries.Result import Result


def DataImpl(file_path: str) -> Result:
    file_path = os.path.join(Config().DATA_DIR, file_path)
    result = Result()
    result.data = File(file_path).load()
    return result



from tep.libraries.Request import Request


class Before:
    def __init__(self):
        self.parametrize: list = []


class After:
    def __init__(self):
        self.extractor: list = []
        self.assertion: list = []
        self.replay: list = []


class Step:
    def __init__(self):
        self.name: str = ""
        self.before: Before = Before()
        self.request: Request = Request()
        self.after: After = After()



class Args:
    @classmethod
    def parse(cls, fields: list, args: tuple, kwargs: dict) -> tuple:
        # Parse fixed args
        results = []
        for i, field in enumerate(fields):
            if i < len(args):
                results.append(args[i])
            else:
                value = kwargs.get(field, None)
                if value:
                    results.append(value)
                    # Args comes from kwargs, pop the key
                    kwargs.pop(field)
        results.append(kwargs)
        return tuple(results)



#!/usr/bin/python
# encoding=utf-8

import jsonpath
from requests import Response


class TepResponse(Response):
    """
    Inherit on requests.Response, adding additional methods
    """

    def __init__(self, response):
        super().__init__()
        for k, v in response.__dict__.items():
            self.__dict__[k] = v

    def jsonpath(self, expr: str):
        """
        Force the first value here for simple values
        If complex values are taken, it is recommended to use jsonpath native directly
        """
        return jsonpath.jsonpath(self.json(), expr)[0]



from loguru import logger


class DB:
    @classmethod
    def pymysql_execute(cls, conn, cursor, sql):
        try:
            cursor.execute(sql)
            conn.commit()
        except Exception as e:
            logger.error(f"Database execute error: {e}")
            conn.rollback()



#!/usr/bin/python
# encoding=utf-8

import json
import os

import yaml


class File:
    def __init__(self, path: str):
        self.path = path

    def load(self) -> [dict, list]:
        file_type = self._file_type()
        if file_type in [".yml", ".yaml", ".YML", "YAML"]:
            return self._yaml_load()
        if file_type in [".json", ".JSON"]:
            return self._json_load()

    def _file_type(self) -> str:
        return os.path.splitext(self.path)[-1]

    def _yaml_load(self) -> [dict, list]:
        with open(self.path, encoding="utf8") as f:
            return yaml.load(f.read(), Loader=yaml.FullLoader)

    def _json_load(self) -> [dict, list]:
        with open(self.path, encoding="utf8") as f:
            return json.load(f)



#!/usr/bin/python
# encoding=utf-8
import inspect
import os
import shutil

import allure_commons
from allure_commons.logger import AllureFileLogger
from allure_pytest.listener import AllureListener
from allure_pytest.plugin import cleanup_factory

from tep.libraries.Config import Config


def tep_plugins():
    """
    Must be placed at the top, execute first to initialize base dir
    """
    caller = inspect.stack()[1]
    Config.BASE_DIR = os.path.abspath(os.path.dirname(caller.filename))
    plugins = _keyword_path() + _fixture_path()  # +[other plugins]
    return plugins


def _tep_reports(config):
    """
    --tep-reports cannot be used together with the allure command line parameter, otherwise an error may occur
    """
    if config.getoption("--tep-reports") and not config.getoption("allure_report_dir"):
        return True
    return False


def _is_master(config):
    """
    During pytest-xdist distributed execution, determine whether it is the master node or a child node
    The main node does not have a workerinput attribute
    config: session.config
    """
    return not hasattr(config, 'workerinput')


def _keyword_path() -> list:
    return ["tep.keywords.api"]


def _fixture_path():
    _fixture_dir = os.path.join(Config.BASE_DIR, "fixture")
    paths = []
    # 项目下的fixtures
    for root, _, files in os.walk(_fixture_dir):
        for file in files:
            if file.startswith("fixture_") and file.endswith(".py"):
                full_path = os.path.join(root, file)
                import_path = full_path.replace(_fixture_dir, "").replace("\\", ".")
                import_path = import_path.replace("/", ".").replace(".py", "")
                paths.append("fixture" + import_path)
    return paths


class Plugin:
    @staticmethod
    def pytest_addoption(parser):
        """
        Allure test report, command line parameters
        """
        parser.addoption(
            "--tep-reports",
            action="store_const",
            const=True,
            help="Create tep allure HTML reports."
        )

    @staticmethod
    def pytest_configure(config):
        """
        Reference: https://github.com/allure-framework/allure-python/blob/master/allure-pytest/src/plugin.py
        In order to generate an allure source file for generating HTML reports
        """
        if _tep_reports(config):
            if os.path.exists(Config.ALLURE_SOURCE_PATH):
                shutil.rmtree(Config.ALLURE_SOURCE_PATH)
            test_listener = AllureListener(config)
            config.pluginmanager.register(test_listener)
            allure_commons.plugin_manager.register(test_listener)
            config.add_cleanup(cleanup_factory(test_listener))

            clean = config.option.clean_alluredir
            file_logger = AllureFileLogger(Config.ALLURE_SOURCE_PATH, clean)  # allure_source
            allure_commons.plugin_manager.register(file_logger)
            config.add_cleanup(cleanup_factory(file_logger))

    @staticmethod
    def pytest_sessionfinish(session):
        """
        Generate an allure report after the test run ends
        """
        reports_path = os.path.join(Config.BASE_DIR, "reports")
        if _tep_reports(session.config):
            if _is_master(session.config):  # Generate reports only at the master node
                # Historical data from the latest report, filling in the allure trend chart
                if os.path.exists(reports_path):
                    his_reports = os.listdir(reports_path)
                    if his_reports:
                        latest_report_history = os.path.join(reports_path, his_reports[-1], "history")
                        shutil.copytree(latest_report_history, os.path.join(Config.ALLURE_SOURCE_PATH, "history"))

                os.system(f"allure generate {Config.ALLURE_SOURCE_PATH} -o {Config().HTML_REPORT_PATH}  --clean")
                shutil.rmtree(Config.ALLURE_SOURCE_PATH)



import difflib
import json
import os
import time

from deepdiff import DeepDiff

from tep.libraries.Sqlite import Sqlite


class Diff:
    current_time = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime(time.time()))
    DIFF_TXT_FILE = "{}.txt".format(current_time)
    DIFF_HTML_FILE = "{}.html".format(current_time)

    @staticmethod
    def make(case_id: str, diff_dir: str):
        results = Sqlite.get_expect_actual(case_id)
        diffs = []
        expect_text = ""
        actual_text = ""
        for row in results:
            expect, actual, method, url = row
            diff = Diff.make_deepdiff(expect, actual)
            diffs.append(diff)
            expect_text += Diff._format_json_str(expect, method, url)
            actual_text += Diff._format_json_str(actual, method, url)
        diff_html = Diff.make_difflib(expect_text, actual_text)
        Diff._output_file(diffs, diff_html, diff_dir)

    @staticmethod
    def make_deepdiff(expect: str, actual: str) -> str:
        diff = DeepDiff(expect, actual)
        return str(diff)

    @staticmethod
    def make_difflib(lines1: str, lines2: str) -> str:
        diff = difflib.HtmlDiff().make_file(lines1.splitlines(), lines2.splitlines())
        return diff

    @staticmethod
    def _format_json_str(text: str, method: str, url: str) -> str:
        s = json.dumps(json.loads(text), indent=4, ensure_ascii=False)
        s = "{} {}\n{}\n\n".format(method, url, s)
        return s

    @staticmethod
    def _output_file(diffs: list, diff_html: str, diff_dir: str):
        with open(os.path.join(diff_dir, Diff.DIFF_TXT_FILE), "w") as f:
            f.write("\n\n".join(diffs))

        styled_html = f"<style>td {{width: 50%;}}</style>\n{diff_html}"
        with open(os.path.join(diff_dir, Diff.DIFF_HTML_FILE), "w") as f:
            f.write(styled_html)



#!/usr/bin/python
# encoding=utf-8

import os

from tep.libraries.Config import Config
from tep.libraries.Cmd import Cmd


class Run:
    def __init__(self, *args, **kwargs):
        os.system(Cmd(*args).pytest())



import base64
import os
import uuid

from haralyzer import HarParser
from loguru import logger

from tep.libraries.Sqlite import Sqlite
from tep.libraries.Step import Step


class Har:
    TEMPLATE = """def test(HTTPRequestKeyword, BodyKeyword):
    var = {var}
    {steps}
"""

    TEMPLATE_IMPORT_REPLAY = """from tep.libraries.Diff import Diff
from tep.libraries.Sqlite import Sqlite


"""

    TEMPLATE_DIFF = """
    Diff.make(var["caseId"], var["diffDir"])
"""

    def __init__(self, har_file: str, profile: dict):
        self.har_file = har_file
        self.profile = profile
        self.replay = profile.get("replay", False)
        filepath = os.path.splitext(self.har_file)[0]
        self.case_file = "{}_test.py".format(filepath)
        self.replay_dff_dir = "{}-replay-diff".format(filepath)
        # Generate a unique ID based on the file path
        self.case_id = str(uuid.uuid5(uuid.UUID("3fa83108-6f0a-4cf0-b687-bbdd294ce7fb"), self.har_file)).replace("-", "")
        self.request_order = 1

    def har2case(self):
        logger.info("Start to generate case")
        self._make_case()
        logger.info("Case generated: {}".format(self.case_file))

    def _make_case(self):
        var = self._prepare_var()
        steps = self._prepare_steps()

        content = Har.TEMPLATE.format(var=var, steps=steps)

        if self.replay:
            content = Har.TEMPLATE_IMPORT_REPLAY + content
            if not os.path.exists(self.replay_dff_dir):
                os.makedirs(self.replay_dff_dir)
            content += Har.TEMPLATE_DIFF

        with open(self.case_file, "w") as f:
            f.write(content)

    def _prepare_var(self) -> str:
        var = {}
        if self.replay:
            var = {
                "caseId": self.case_id,
                "requestOrder": 1,
                "diffDir": self.replay_dff_dir
            }
        return str(var)

    def _prepare_steps(self) -> str:
        steps = []
        har_parser = HarParser.from_file(self.har_file)
        for page in har_parser.pages:
            for entry in page.entries:
                step = self._prepare_step(entry)
                steps += step
        return "\n    ".join(steps)

    def _prepare_step(self, entry) -> list:
        logger.info("{} {} Convert step", entry.request.method, entry.request.url)
        step = Step()
        self._make_request_method(step, entry)
        self._make_request_url(step, entry)
        self._make_request_headers(step, entry)
        self._make_request_body(step, entry)
        self._make_before_param(step, entry)
        self._make_after_extract(step, entry)
        self._make_after_assert(step, entry)

        if self.replay:
            self._make_after_replay(step, entry)
            self._save_replay(step, entry)  # Save replay data to sqlite

        return self._make_statement(step, entry)

    def _make_request_method(self, step, entry):
        step.request.method = entry.request.method

    def _make_request_url(self, step, entry):
        step.request.url = entry.request.url

    def _make_request_headers(self, step, entry):
        headers = entry.request.headers
        cookies = entry.request.cookies
        h = dict()
        for header in headers:
            h[header["name"]] = header["value"]
        for cookie in cookies:
            h[cookie["name"]] = cookie["value"]
        step.request.headers = h

    def _make_request_body(self, step, entry):
        step.request.body = entry.request.text

    def _make_before_param(self, step, entry):
        stmt = [
            '',  # Blank line, distinguishing paragraphs
            'url = "{}"'.format(step.request.url),
            'headers = {}'.format(step.request.headers),
        ]
        body_stmt = [
            'body = r"""{}"""'.format(step.request.body),
            'ro = BodyKeyword(body)',
            '# ro = BodyKeyword(body, {"$.jsonpath": "value"})',
            'body = ro.data'
        ]
        if step.request.body:
            stmt += body_stmt
        step.before.parametrize = stmt

    def _make_after_extract(self, step, entry):
        stmt = [
            '# user_defined_var = ro.response.jsonpath("$.jsonpath")'
        ]
        step.after.extractor = stmt

    def _make_after_assert(self, step, entry):
        stmt = [
            'assert ro.response.status_code < 400'
        ]
        step.after.assertion = stmt

    def _make_after_replay(self, step, entry):
        stmt = [
            'Sqlite.record_actual((ro.response.text, var["caseId"], var["requestOrder"], "{}", "{}"), var)'.format(step.request.method, step.request.url)
        ]
        step.after.replay = stmt

    def _make_statement(self, step, entry) -> list:
        stmt = []
        stmt += step.before.parametrize
        stmt += [
            'ro = HTTPRequestKeyword({})'.format(self._request_param(step, entry))
        ]
        stmt += step.after.extractor
        stmt += step.after.assertion
        stmt += step.after.replay
        return stmt

    def _request_param(self, step, entry) -> str:
        param = ""
        mime_type = entry.request.mimeType
        b = 'data=body' if mime_type and mime_type.startswith("application/x-www-form-urlencoded") else 'json=body'
        if step.request.method == "GET":
            param = '"get", url=url, headers=headers, params=body' if step.request.body else '"get", url=url, headers=headers'
        if step.request.method == "POST":
            param = '"post", url=url, headers=headers, '
            param += b
        if step.request.method == "PUT":
            param = '"put", url=url, headers=headers, '
            param += b
        if step.request.method == "DELETE":
            param = '"delete", url=url, headers=headers'
        if self.profile.get("http2", False):
            param += ', http2=True'
        return param

    def _save_replay(self, step, entry):
        Sqlite.create_table_replay()
        data = (
            self.case_id,
            self.request_order,
            step.request.method,
            step.request.url,
            self._decode_text(entry)
        )
        Sqlite.insert_into_replay_expect(data)
        self.request_order += 1

    def _decode_text(self, entry):
        text = entry.response.text
        if text:
            mime_type = entry.response.mimeType
            if mime_type and mime_type.startswith("application/json"):
                encoding = entry.response.textEncoding
                if encoding and encoding == "base64":
                    content = base64.b64decode(text).decode('utf-8')
                    return content
        return text



import os.path
import time

from loguru import logger

from tep.libraries.Config import Config


class Log:
    current_time = time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(time.time()))
    log_dir = os.path.join(Config.BASE_DIR, "log")
    if not os.path.exists(log_dir):
        os.mkdir(log_dir)

    @staticmethod
    def logger():
        logger.add(os.path.join(Log.log_dir, "{}.log".format(Log.current_time)), filter=lambda record: record["extra"].get("name") == "user")
        log = logger.bind(name="user")
        return log

    @staticmethod
    def sys_logger():
        logger.add(os.path.join(Log.log_dir, "sys_{}.log".format(Log.current_time)), filter=lambda record: record["extra"].get("name") == "sys")
        log = logger.bind(name="sys")
        return log



import pymysql

from tep.libraries.Result import Result


def DbcImpl(host: str, port: int, user: str, password: str, database: str) -> Result:
    conn = pymysql.connect(host=host, port=port, user=user, password=password, database=database)
    result = Result()
    result.conn = conn
    return result



import os
import sqlite3

from tep.libraries.Config import Config


class Sqlite:
    DB_FILE = "sqlite3.db"

    @staticmethod
    def execute(sql: str, data: tuple = None):
        os.chdir(Config.BASE_DIR)
        conn = sqlite3.connect(Sqlite.DB_FILE)
        if data:
            conn.execute(sql, data)
        else:
            conn.execute(sql)
        conn.commit()
        conn.close()

    @staticmethod
    def create_table_replay():
        Sqlite.execute("""CREATE TABLE IF NOT EXISTS replay
                  (id INTEGER PRIMARY KEY AUTOINCREMENT,
                   case_id TEXT NOT NULL,
                   request_order INTEGER NOT NULL,
                   method TEXT NOT NULL,
                   url TEXT NOT NULL,
                   expect TEXT NOT NULL,
                   actual TEXT)""")

    @staticmethod
    def insert_into_replay_expect(data: tuple):
        if not Sqlite.is_replay_existed(data):
            Sqlite.execute("INSERT INTO replay(case_id, request_order, method, url, expect) VALUES (?, ?, ?, ?, ?)", data)

    @staticmethod
    def is_replay_existed(data: tuple) -> bool:
        os.chdir(Config.BASE_DIR)
        conn = sqlite3.connect(Sqlite.DB_FILE)
        cursor = conn.cursor()
        where = data[:-1]
        cursor.execute("SELECT * FROM replay WHERE case_id = ? AND request_order = ? AND method = ? AND url = ?", where)
        results = cursor.fetchall()
        conn.close()
        if results:
            return True
        return False

    @staticmethod
    def update_replay_actual(data: tuple):
        Sqlite.execute("UPDATE replay SET actual = ? WHERE case_id = ? AND request_order = ? AND method = ? AND url = ?", data)

    @staticmethod
    def record_actual(data: tuple, var: dict):
        Sqlite.update_replay_actual(data)
        var["requestOrder"] += 1


    @staticmethod
    def get_expect_actual(case_id: str):
        os.chdir(Config.BASE_DIR)
        conn = sqlite3.connect(Sqlite.DB_FILE)
        cursor = conn.cursor()
        cursor.execute("SELECT expect, actual, method, url FROM replay WHERE case_id = ?", (case_id,))
        results = cursor.fetchall()
        conn.close()
        return results



#!/usr/bin/python
# encoding=utf-8
import os.path

from loguru import logger

from tep.libraries.Config import Config


class Cmd:
    template = "pytest -s {where_to_run} {tep_report}"

    def __init__(self, *args, **kwargs):
        settings = args[0]
        self.RUN_PATH = [os.path.join(Config().CASE_DIR, path) for path in settings["path"]]
        self.RUN_REPORT = settings["report"]
        self.RUN_REPORT_TYPE = settings["report_type"]

    def pytest(self) -> str:
        cmd = self.template.format(
            where_to_run=" ".join(self.RUN_PATH),
            tep_report=self.tep_report()
        )
        return cmd

    def tep_report(self) -> str:
        if self.RUN_REPORT:
            if self.RUN_REPORT_TYPE == "pytest-html":
                return f"--html={Config().HTML_REPORT_PATH}.html --self-contained-html"
            elif self.RUN_REPORT_TYPE == "allure":
                return "--tep-reports"
        return ""



#!/usr/bin/python
# encoding=utf-8

import argparse
import sys

from tep import __description__, __version__
from tep.scaffold import init_parser_scaffold, scaffold


def main():
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument("-V", "--version", dest="version", action="store_true", help="show version")
    subparsers = parser.add_subparsers(help="sub-command help")
    sub_parser_scaffold = init_parser_scaffold(subparsers)

    if len(sys.argv) == 1:
        # tep
        parser.print_help()
        sys.exit(0)
    elif len(sys.argv) == 2:
        if sys.argv[1] in ["-V", "--version"]:
            print(f"Current Version: V{__version__}")
            print(r"""
 ____o__ __o____   o__ __o__/_   o__ __o
  /   \   /   \   <|    v       <|     v\
       \o/        < >           / \     <\
        |          |            \o/     o/
       < >         o__/_         |__  _<|/
        |          |             |
        o         <o>           <o>
       <|          |             |
       / \        / \  _\o__/_  / \
""")
        elif sys.argv[1] in ["-h", "--help"]:
            parser.print_help()
        elif sys.argv[1] == "new":
            sub_parser_scaffold.print_help()
        sys.exit(0)

    args = parser.parse_args()

    if sys.argv[1] == "new":
        scaffold(args)



import pytest

from tep.keywords.impl.BodyImpl import BodyImpl
from tep.keywords.impl.DataImpl import DataImpl
from tep.keywords.impl.DbcImpl import DbcImpl
from tep.keywords.impl.HTTPRequestImpl import HTTPRequestImpl
from tep.keywords.impl.UserDefinedVariablesImpl import UserDefinedVariablesImpl
from tep.libraries.Args import Args
from tep.libraries.Result import Result

"""
Adaptation Layer, accept any parameter, return Result object，backward compatible
"""


@pytest.fixture(scope="session")
def HTTPRequestKeyword():
    def _function(*args, **kwargs) -> Result:
        method, url, kwargs = Args.parse(["method", "url"], args, kwargs)
        return HTTPRequestImpl(method, url, **kwargs)

    return _function


@pytest.fixture(scope="session")
def BodyKeyword():
    def _function(*args, **kwargs) -> Result:
        try:
            json_str, expr, kwargs = Args.parse(["json_str", "expr"], args, kwargs)
        except Exception:
            json_str, kwargs = Args.parse(["json_str", "expr"], args, kwargs)
            expr = None
        return BodyImpl(json_str, expr)

    return _function


@pytest.fixture(scope="session")
def UserDefinedVariablesKeyword():
    def _function(*args, **kwargs) -> Result:
        return UserDefinedVariablesImpl()

    return _function


@pytest.fixture(scope="session")
def DataKeyword():
    def _function(*args, **kwargs) -> Result:
        file_path, kwargs = Args.parse(["file_path"], args, kwargs)
        return DataImpl(file_path)

    return _function


@pytest.fixture(scope="session")
def DbcKeyword():
    def _function(*args, **kwargs) -> Result:
        host, port, user, password, database, kwargs = Args.parse(["host", "port", "user", "password", "database"], args, kwargs)
        return DbcImpl(host, port, user, password, database)

    return _function



#!/usr/bin/python
# encoding=utf-8

import json
import re
from typing import Any

from tep.libraries.Result import Result


def BodyImpl(json_str: str, expr: dict = None) -> Result:
    json_obj = json.loads(json_str)
    if expr:
        for json_path, value in expr.items():
            _assign(json_obj, json_path, value)
    result = Result()
    result.data = json_obj
    return result


def _jsonpath_to_dict_expr(jsonpath: str) -> str:
    """
    Input: $.store.book[0].title
    Output: '["store"]["book"][0]["title"]'
    """
    tokens = re.findall(r'\.(\w+)|\[(\d+)\]', jsonpath)
    expr = ''
    for token in tokens:
        if token[0]:
            expr += '["{}"]'.format(token[0])
        else:
            expr += '[{}]'.format(token[1])
    return expr


def _parse_dict_expr(expr: str) -> list:
    """
    Input: '["store"]["book"][0]["title"]'
    Output: ['store', 'book', 0, 'title']
    """
    tokens = re.findall(r'\["(.*?)"\]|\[(\d+)\]', expr)
    result = [int(index) if index.isdigit() else name for name, index in tokens]
    return result


def _nested_modify(json_obj: [dict, list], keys: list, value: Any, current_level: int = 0):
    if current_level == len(keys) - 1:
        json_obj[keys[current_level]] = value
    else:
        current_key = keys[current_level]
        # Nested string json {"id": 1, "param": "{\"page\": 1}"}
        if isinstance(json_obj[current_key], str):
            # str to json
            current_value = json.loads(json_obj[current_key])
            if isinstance(current_value, dict) or isinstance(current_value, list):
                nested_string_json_obj = current_value
                _nested_modify(nested_string_json_obj, keys[current_level + 1:], value)
                # json to str
                json_obj[current_key] = json.dumps(nested_string_json_obj, ensure_ascii=False)
        else:
            _nested_modify(json_obj[current_key], keys, value, current_level + 1)


def _assign(json_obj: [dict, list], json_path: str, value: Any):
    dict_expr = _jsonpath_to_dict_expr(json_path)
    keys = _parse_dict_expr(dict_expr)
    _nested_modify(json_obj, keys, value)



#!/usr/bin/python
# encoding=utf-8

import os
import platform
import sys

from loguru import logger

from tep.libraries.Config import Config


def init_parser_scaffold(subparsers):
    sub_parser_scaffold = subparsers.add_parser("new", help="Create a new project with template structure.")
    sub_parser_scaffold.add_argument("project_name", type=str, nargs="?", help="Specify new project name.")
    sub_parser_scaffold.add_argument(
        "-venv",
        dest="create_venv",
        action="store_true",
        help="Create virtual environment in the project, and install tep.",
    )
    return sub_parser_scaffold


def scaffold(args):
    Config.CREATE_ENV = args.create_venv
    sys.exit(create_scaffold(args.project_name))


def create_scaffold(project_name):
    if os.path.isdir(project_name):
        logger.warning(f"Project folder {project_name} exists, please specify a new project name")
        return 1
    elif os.path.isfile(project_name):
        logger.warning(f"Project name {project_name} conflicts with existed file, please specify a new one")
        return 1

    print(f"Create new project: {project_name}")
    print(f"Project root dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"Created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"Created file:   {path}"
        print(msg)

    create_folder(project_name)
    create_folder(os.path.join(project_name, "case"))
    create_folder(os.path.join(project_name, "data"))
    create_folder(os.path.join(project_name, "report"))

    replay_content = """import os

from tep.libraries.Config import Config
from tep.libraries.Har import Har

if __name__ == '__main__':
    har_file = os.path.join(Config.BASE_DIR, "case", "replay_demo.har")
    profile = {}
    Har(har_file, profile).har2case()
"""
    create_file(os.path.join(project_name, "replay.py"), replay_content)
    run_content = """from tep.libraries.Run import Run

if __name__ == '__main__':
    settings = {
        "path": ["test_demo.py"],  # Path to run, relative path to case
        "report": False,  # Output test report or not
        "report_type": "pytest-html"  # "pytest-html" "allure"
    }
    Run(settings)
"""
    create_file(os.path.join(project_name, "run.py"), run_content)
    conftest_content = """from tep.plugin import tep_plugins

pytest_plugins = tep_plugins()
"""
    create_file(os.path.join(project_name, "conftest.py"), conftest_content)
    create_file(os.path.join(project_name, "pytest.ini"), "")
    gitignore_content = """.idea
.pytest_cache/
__pycache__/
.har
"""
    create_file(os.path.join(project_name, ".gitignore"), gitignore_content)
    create_file(os.path.join(project_name, "case", "__init__.py"), "")
    demo_content = """def test(HTTPRequestKeyword):
    ro = HTTPRequestKeyword("get", url="http://httpbin.org/status/200")
    assert ro.response.status_code == 200
"""
    create_file(os.path.join(project_name, "case", "test_demo.py"), demo_content)
    user_defined_variables_content = 'name: "公众号测试开发刚哥"'
    create_file(os.path.join(project_name, "data", "UserDefinedVariables.yaml"), user_defined_variables_content)

    if Config.CREATE_ENV:
        # Create Python virtual Environment
        os.chdir(project_name)
        print("\nCreating virtual environment")
        os.system("python -m venv .venv")
        print("Created virtual environment: .venv")

        # Install tep in the Python virtual Environment
        print("Installing tep")
        if platform.system().lower() == 'windows':
            os.chdir(".venv")
            os.chdir("Scripts")
            os.system("pip install tep")
        elif platform.system().lower() == 'linux':
            os.chdir(".venv")
            os.chdir("bin")
            os.system("pip install tep")



#!/usr/bin/python
# encoding=utf-8

import os
import time


class Config:
    # Class variable initialize first
    BASE_DIR = ""

    # Constant
    CREATE_ENV = False
    # The temporary directory of the allure source file, which is a pile of JSON files,
    # will be deleted when generating HTML reports
    ALLURE_SOURCE_PATH = ".allure.source.temp"

    def __init__(self):
        # Instance variable initialize after class variable assigned
        self.CASE_DIR = os.path.join(self.BASE_DIR, "case")
        self.DATA_DIR = os.path.join(self.BASE_DIR, "data")
        self.REPORT_DIR = os.path.join(self.BASE_DIR, "report")

        current_time = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime(time.time()))
        self.HTML_REPORT_PATH = os.path.join(self.REPORT_DIR, "report-" + current_time)




from typing import Any, List, Union

from httpx import Response

from tep.libraries.TepResponse import TepResponse


class Result:
    # Http request, response
    response: List[Union[TepResponse, Response]] = None
    # Any data
    data: Any = None
    # Connect database, connection
    conn = None
    # Connect database, cursor
    cursor = None
